CONTENTS OF THIS FILE
---------------------

 * Introduction
 * Requirements
 * Installation
 * Configuration
 * Transmuter Plugins
 * Maintainers


INTRODUCTION
------------

The module name is a play on the alchemy transmutation of elements. Elements in
this case refers to **HTML elements** or **theme elements**

Adds transmuter plugins which can provide theme preprocess functionality for
Drupal templates. Transmuter plugins can be discovered by using the Transmuter
attribute on plugin classes in both modules and theme
[@see Transmuter Plugins](#transmuter-plugins).

Attribute discovery of theme defined plugins are possible with the use of the
[themespace](https://www.drupal.org/project/themespace) module.

Transmuter plugins are added to the theme registry and are compatible and work
well with theme preprocess hooks which are defined by other modules and themes.
By applying to the theme registry, the definitions are cached when the theme
registry is built.

Because the hooks are applied to the registry and cached, the performance of
transmuter preprocess plugins is very good and has low overhead.

 * For a full description of the module, visit the project page:
   https://www.drupal.org/project/transmuter

 * To submit bug reports and feature suggestions, or track changes:
   https://www.drupal.org/project/issues/transmuter


REQUIREMENTS
------------

This module requires the following modules:

 * [Themespace](https://www.drupal.org/project/themespace)


INSTALLATION
------------

Install as you would normally install a contributed Drupal module. Visit
https://www.drupal.org/node/1897420 for further information.


CONFIGURATION
-------------

This module requires no extra configurations.


TRANSMUTER PLUGINS
----------------------

Create transmuter plugins with the attribute
`Drupal\transmuter\Attribute\Transmuter` attribute in the
`<module>\Plugin\Transmuter`, or `<theme>\Theme\Plugin\Transmuter` namespace.
Transmuters plugins should implement
`Drupal\transmuter\Plugin\TransmuterInterface`.


### Attribute and Plugin Definition

Ensure that plugin classes implement the `TransmuterInterface` and is in the
`Plugin\Transmuter` folder and namespace of the defining module or theme, and
the class has the `Transmuter` attribute.

**Below is an example of a transmuter plugin class:**
```
<?php

namespace <theme>\Theme\Plugin\Transmuter;

use Drupal\Core\Plugin\PluginBase;
use Drupal\transmuter\Attribute\Transmuter;
use Drupal\transmuter\Plugin\TransmuterInterface;

/**
 * Example transmuter plugin defined by a theme.
 */
#[Transmuter(
  id: 'theme_example_transmuter',
  // Single hooks can be a string
  hook: [
    'node__page',
    'node__article',
  ],
  priority: 100 // optional
  deriver: MyTransmuterDeriver::class // optional, @see plugin deriver
)]
class MyTransmuter extends PluginBase implements TransmuterInterface {

  /**
   * {@inheritdoc}
   */
  public function preprocess(array &$variables, $hook, array $info = []): void {
    ...
  }

}
```

The `TransmuterInterface::preprocess()` is the only method and is designed to
resemble a preprocess hook callback. It is recommended to use the `Drupal\Core\Plugin\ContainerFactoryPluginInterface` when Drupal
container service inject is needed.

### Theme Defined Transmuters

Transmuter plugins which are defined by themes are **cached per theme** and will
only be applied when that theme is active. This behavior matches what theme
preprocess hooks defined by themes would do.

Transmuters are applied to when sub-themes are active as well, and will run
execute before the sub-theme definitions.

@see \Drupal\transmuter\ThemeRegistryBuilder::doAlter


MAINTAINERS
-----------

Current maintainers:
 * Liem Khuu (lemming) - https://www.drupal.org/u/lemming
